@@ -37,6 +37,10 @@ typedef struct _sem {
 	int		level;
 	int		tid;
 	sem_t		s;
+	char *		lock_file;
+	int		lock_line;
+	char *		unlock_file;
+	int		unlock_line;
 } _SEM;
 
 #endif
