@@ -29,6 +29,7 @@
 #include	"utils.h"
 #include	"lock_level.h"
 #include	"pri_level.h"
+#include	"machine/fork_lock.h"
 
 extern S_TABLE s_file_table;
 extern SEM stream_lock;
@@ -154,6 +155,7 @@ int ret_errno;
 	er = pipe(p);
 retry:
 	errno = 0;
+	wlock_fork();
 	id = fork();
 	if ( id == 0 ) {
 		/* child process */
@@ -222,6 +224,7 @@ retry:
 		er = execvp(str,argv);
 		exit(1);
 	}
+	wunlock_fork();
 	if ( id < 0 && errno == EAGAIN ) {
 		sleep_sec(1);
 		goto retry;
