@@ -215,6 +215,8 @@ SEM s;
 }
 #endif
 
+
+
 void
 init_task(int flags)
 {
@@ -243,6 +245,8 @@ int i;
 
 	setup_main_thread();
 	task_init = 2;
+
+	init_fork_lock();
 }
 
 
@@ -511,8 +515,10 @@ _get_tid()
 void
 _insert_task_hash(TASK_BLOCK * b)
 {
-unsigned int key;
+int key;
 	key = b->hard_tid % TASK_BLOCK_HASH;
+	if ( key < 0 )
+		key = -key;
 	b->next = task_hash[key];
 	task_hash[key] = b;
 
@@ -527,9 +533,11 @@ unsigned int key;
 void
 _delete_task_hash(TASK_BLOCK * b)
 {
-unsigned int key;
+int key;
 TASK_BLOCK ** bp;
 	key = b->hard_tid % TASK_BLOCK_HASH;
+	if ( key < 0 )
+		key = -key;
 	bp = &task_hash[key];
 	for ( ; *bp ; bp = &(*bp)->next )
 		if ( *bp == b ) {
@@ -600,6 +608,8 @@ int er;
 
 		(*b.func)(b.data);
 
+		close_thread_area();
+
 		lock_task(task_lock);
 		b.soft_tid = 0;
 		b.finish_time = get_xltime();
@@ -662,6 +672,8 @@ TASK_BLOCK * b;
 	hard_tid = _get_tid();
 	lock_task(task_lock);
 	key = hard_tid % TASK_BLOCK_HASH;
+	if ( key < 0 )
+		key = -key;
 	for ( b = task_hash[key] ; b ; b = b->next )
 		if ( b->hard_tid == hard_tid ) {
 			soft_tid = b->soft_tid;
