@@ -34,6 +34,7 @@
 #include	<io.h>
 #include	"flush_pipe.h"
 
+extern STREAM * stream_list;
 extern SEM stream_lock;
 STREAM * s_error_stream();
 int s_close_fpipe();
@@ -71,16 +72,17 @@ s_open_fpipe(STREAM * p[2])
 {
 extern void _s_open(STREAM * s,int mode);
 
-int pfd[2];
+int pfd1[2], pfd2[2];
 
-	if ( _pipe(pfd, 1024, O_BINARY ) < 0 )
+	if ( _pipe(pfd1, 1024, O_BINARY ) < 0 || _pipe(pfd2, 1024, O_BINARY ) < 0 )
 		return -1;
 
 	errno = 0;
 	p[0] = d_alloc(sizeof(S_FPIPE),134);
 	p[0]->h.tbl = &s_fpipe_table;
 	p[0]->h.thread = 0;
-	p[0]->fpipe.fid = pfd[0];
+	p[0]->fpipe.fid_read = pfd1[0];
+	p[0]->fpipe.fid_write = pfd2[1];
 	p[0]->fpipe.head = p[0]->fpipe.tail = 0;
 	p[0]->fpipe.read_flag = 0;
 	p[0]->fpipe.flags = 0;
@@ -88,8 +90,9 @@ int pfd[2];
 
 	p[1] = d_alloc(sizeof(S_FPIPE),134);
 	p[1]->h.tbl = &s_fpipe_table;
-	p[1]->h.thread = 0;
-	p[1]->fpipe.fid = pfd[1];
+	p[1]->h.thread = 0;	
+	p[1]->fpipe.fid_read = pfd2[0];
+	p[1]->fpipe.fid_write = pfd1[1];
 	p[1]->fpipe.head = p[1]->fpipe.tail = 0;
 	p[1]->fpipe.read_flag = 0;
 	p[1]->fpipe.flags = 0;
@@ -110,7 +113,8 @@ PIPE_READ_BUF * b;
 void _s_close_sync(STREAM * s);
 	
 	_s_close_sync(s);
-	close(s->fpipe.fid);
+	close(s->fpipe.fid_read);
+	close(s->fpipe.fid_write);
 	for ( ; s->fpipe.head ; ) {
 		b = s->fpipe.head;
 		s->fpipe.head = b->next;
@@ -205,7 +209,7 @@ int ip;
 	s->fpipe.read_flag;
 	unlock_task(lock,"read_phy_pipe");
 
-	ret = rw_loop(read,s,s->fpipe.fid,&len,sizeof(len));
+	ret = rw_loop(read,s,s->fpipe.fid_read,&len,sizeof(len));
 	if ( ret != sizeof(len) ) {
 		if ( lock )
 			lock_task(lock);
@@ -223,11 +227,11 @@ int ip;
 		case FPC_GET_IP:
 			if ( s->fpipe.this_ip_addr == 0 )
 				ip = -1;
-			rw_loop(write,s,s->fpipe.fid,
+			rw_loop(write,s,s->fpipe.fid_write,
 				&ip,sizeof(ip));
 			break;
 		case FPC_GET_IP_REP:
-			rw_loop(read,s,s->fpipe.fid,
+			rw_loop(read,s,s->fpipe.fid_read,
 				&s->fpipe.peer_ip_addr,sizeof(ip));
 			break;
 		default:
@@ -241,7 +245,7 @@ int ip;
 	ptr = (char*)(b+1);
 	b->ptr = 0;
 	b->len = len;
-	ret = rw_loop(read,s,s->fpipe.fid,ptr,len);
+	ret = rw_loop(read,s,s->fpipe.fid_read,ptr,len);
 	if ( ret != len ) {
 		d_f_ree(b);
 
@@ -342,7 +346,7 @@ int len;
 int ret;
 	len = -1;
 	lock_task(stream_lock);
-	ret = rw_loop(write,s,s->fpipe.fid,&len,sizeof(len));
+	ret = rw_loop(write,s,s->fpipe.fid_write,&len,sizeof(len));
 	unlock_task(stream_lock,"s_flush_fpipe");
 	if ( ret < 0 )
 		return -1;
@@ -357,7 +361,8 @@ s_proc_send_fpipe(STREAM * s,PROC_SEND * ps)
 		return -1;
 	ps->msg[0] = s->h.tbl->type;
 	ps->msg[1] = 0;
-	ps->fid[0] = s->fpipe.fid;
+	ps->fid[0] = s->fpipe.fid_read;
+	ps->fid[1] = s->fpipe.fid_write;
 	ps->fid[1] = -1;
 	return 0;
 }
@@ -372,23 +377,39 @@ int mode;
 
 	pr->ptr ++;
 
-	fid = pr->fidp ++;
-
-/*
-	mode = fcntl(fid,F_GETFL);
-	if ( mode < 0 )
-		return 0;
-	mode &= ~3;
-	fcntl(fid,F_SETFL,mode);
-*/
 	mode = O_RDONLY;
 	ret = d_alloc(sizeof(S_FPIPE),133);
 	ret->h.tbl = pr->tbl;
 	ret->h.thread = 0;
-	ret->fpipe.fid = fid;
 	ret->fpipe.head = ret->fpipe.tail = 0;
 	ret->fpipe.read_flag = 0;
 
+	fid = pr->fidp ++;
+/* 
+	// winapi dose not have fctrl.
+	mode = fcntl(fid,F_GETFL);
+	if ( mode < 0 ) {
+		d_f_ree(ret);
+		return 0;
+	}
+	mode &= ~3;
+	fcntl(fid,F_SETFL,mode);
+*/
+	ret->fpipe.fid_read = fid;
+
+	fid = pr->fidp ++;
+/*
+	// winapi dose not have fctrl.
+	mode = fcntl(fid,F_GETFL);
+	if ( mode < 0 ) {
+		d_f_ree(ret);
+		return 0;
+	}
+	mode &= ~3;
+	fcntl(fid,F_SETFL,mode);
+*/
+	ret->fpipe.fid_write = fid;
+
 	_s_open(ret,mode);
 	return ret;
 }
@@ -444,7 +465,7 @@ int cmd;
 		return s->fpipe.peer_ip_addr;
 	s->fpipe.read_flag = 1;
 	cmd = FPC_GET_IP;
-	if ( rw_loop(write,s,s->fpipe.fid,&cmd,sizeof(cmd)) <= 0 )
+	if ( rw_loop(write,s,s->fpipe.fid_write,&cmd,sizeof(cmd)) <= 0 )
 		return -1;
 	for ( ; ; ) {
 		if ( read_phy_pipe(s,stream_lock) < 0 ) {
