@@ -1050,6 +1050,15 @@ end:
 }
 
 
+void
+bomb()
+{
+/*
+	gc_tick_notin_tick();
+*/
+}
+
+
 void
 _load_resource_task0(TKEY d)
 {
@@ -1089,8 +1098,10 @@ XL_SEXP * ret, * last_tag;
 			er_panic("resource");
 		switch ( wp->option ) {
 		case LRO_FULL:
+bomb();
 			if ( get_rs_resource_cache(&rsb,
 					get_url_str2(&wp->url)) < 0 ) {
+bomb();
 /*
 ss_printf("GET EMPTY %ls\n",get_url_str2(&wp->url));
 */
@@ -1100,6 +1111,7 @@ ss_printf("GET EMPTY %ls\n",get_url_str2(&wp->url));
 /*
 ss_printf("GET SUCCESS %ls\n",get_url_str2(&wp->url));
 */
+bomb();
 			if ( rsb.flags & RSBF_MAP ) {
 ss_printf("MAP--------\n");
 				rs_cache_setup_c_unit(
@@ -1723,8 +1735,9 @@ _resource_cache_invoke(RESOURCE * r)
 RS_BUF b;
 	b.r = r;
 	b.sexp = r->h.data;
-
+bomb();
 	set_rs_resource_cache(&b);
+bomb();
 
 	r->h.data = 0;
 }
