@@ -58,12 +58,9 @@ void _flush_route(MP_ROUTE *);
 void err_route_tick();
 
 SYS_QUEUE	rr_que1,rr_que2;
-int cache_start;
 
 MP_ROUTE_T * wait_list;
 
-void rr_task_cache();
-
 void
 init_mp_cache(MP_CACHE * c)
 {
@@ -113,7 +110,6 @@ void gc_gb_sexp();
 	rr_que2.pri = PRI_FETCH;
 	setup_queue(&rr_que2);
 
-	create_task(rr_task_cache,0,0);
 	new_tick(err_route_tick,10,0);
 }
 
@@ -176,8 +172,7 @@ PURGE_URL * pm;
 		purge_url_tail = pm;
 	}
 	else	purge_url_tail = purge_url_head = pm;
-	cache_start = 1;
-	wakeup_task((int)&cache_start);
+	rcache_invoke();
 }
 
 void
@@ -711,8 +706,7 @@ XL_SEXP * rec;
 	r->sts = RS_OK;
 	r->flags |= RF_DIRTY;
 	free_mp_path(old);
-	cache_start = 1;
-	wakeup_task((int)&cache_start);
+	rcache_invoke();
 	return r;
 err:
 	r->sts = RS_ERR;
@@ -940,7 +934,6 @@ MP_ROUTE * ret,* new_r;
 
 	unlock_task(route_lock,"_get_in_file");
 	ret = get_rt_cache(start,target);
-printf("GET RT %x\n",ret);
 	lock_task(route_lock);
 	if ( ret == 0 )
 		return 0;
@@ -1420,59 +1413,51 @@ typedef struct mp_list {
 } MP_LIST;
 
 void
-rr_task_cache()
+routing_cache_invoke()
 {
 MP_ROUTE * mp;
 int f;
 MP_LIST * lst, * el;
 L_CHAR * map;
-	for ( ; ; ) {
-		lock_task(route_lock);
-		for ( ; cache_start == 0 ; ) {
-			sleep_task((int)&cache_start,route_lock);
-			lock_task(route_lock);
-		}
-		cache_start = 0;
-		lst = 0;
-		for ( mp = direct.root->next ;
-				mp != direct.root ;
-				mp = mp->next ) {
-			if ( mp->sts != RS_OK )
-				continue;
-			if ( !(mp->flags & RF_DIRTY) )
-				continue;
-			mp->flags &= ~RF_DIRTY;
-			el = d_alloc(sizeof(*el),22);
-			el->mp = mp;
-			el->next = lst;	
-			lst = el;
-			mp->lock ++;
-		}
-		unlock_task(route_lock,"rr_task_cache");
-
-		for ( ; lst ; ) {
-			el = lst;
-printf("SET RT %ls %ls\n",el->mp->head->url,el->mp->tail->url);
-			set_rt_cache(el->mp);
-
-			flush_route(el->mp);
-		
-			lst = el->next;
-			d_f_ree(el);
-		}
-		flush_rt_cache();
-
-		sleep_sec(1);
-
-		for ( ; ; ) {
-			map = delete_purge_url();
-			if ( map == 0 )
-				break;
-			purge_rcache(map);
-			d_f_ree(map);
-		}
-		flush_rt_cache();
+	lock_task(route_lock);
+	lst = 0;
+	for ( mp = direct.root->next ;
+			mp != direct.root ;
+			mp = mp->next ) {
+		if ( mp->sts != RS_OK )
+			continue;
+		if ( !(mp->flags & RF_DIRTY) )
+			continue;
+		mp->flags &= ~RF_DIRTY;
+		el = d_alloc(sizeof(*el),22);
+		el->mp = mp;
+		el->next = lst;	
+		lst = el;
+		mp->lock ++;
 	}
+	unlock_task(route_lock,"rr_task_cache");
+
+	for ( ; lst ; ) {
+		el = lst;
+		set_rt_cache(el->mp);
+
+		flush_route(el->mp);
+		
+		lst = el->next;
+		d_f_ree(el);
+	}
+	flush_rt_cache();
+
+	sleep_sec(1);
+
+	for ( ; ; ) {
+		map = delete_purge_url();
+		if ( map == 0 )
+			break;
+		purge_rcache(map);
+		d_f_ree(map);
+	}
+	flush_rt_cache();
 }
 
 MP_ROUTE *
@@ -1544,9 +1529,9 @@ int sts;
 		goto end;
 	}
 	_wait_mp_route(n);
-
+/*
 ss_printf("STS %x %x\n",n,n->flags);
-
+*/
 	if ( (n->flags & RQT_ROUTE_RES_MASK) == RQT_ROUTE_ERR )
 		ret = MP_NO_ROUTE;
 	else	ret = n->route;
