@@ -33,9 +33,9 @@ VERSION xl_lang_version = {
 VERSION xl_proto_version = {
 	"XL.proto",
 	"XL",
-	"(C) 2000-2002 Hirohisa Mori",
-	"ver.A.01",
-	2002,5,27,
+	"(C) 2000-2003 Hirohisa Mori",
+	"ver.A.02.09",
+	2002,9,19,
 	0,
 	"XL protocol version"
 };
@@ -45,8 +45,8 @@ VERSION xl_code_version = {
 	"XL.code",
 	"XL",
 	"(C) 2000-2003 Hirohisa Mori",
-	"ver.A.02.09 (ASTROBOY)",
-	2003,9,18,
+	"ver.A.02.10 (ASTROBOY)",
+	2003,9,19,
 	0,
 	"version of XL code (src/xl)"
 };
