@@ -207,9 +207,7 @@ XL_INTERPRETER * my_xli;
 void gc_gb_sexp();
 CODE_METHOD * cm1;
 
-int tim,_tim,tim1,tim2,tim3,tim4;
 
-set_cpu_msg(98141);
 
 
 	my_xli = _get_my_xli();
@@ -242,7 +240,6 @@ set_cpu_msg(98141);
 	resp->remote_seq = resp->line;
 
 	resp->cur_env = env;
-set_cpu_msg(98142);
 	if ( xli == 0 ) {
 		ret = get_error(
 			resp->file,
@@ -263,7 +260,6 @@ set_cpu_msg(98142);
 			n_get_string("invalid id"));
 		return ret;
 	}
-set_cpu_msg(98143);
 
 	_lock_xli_out(xli);
 
@@ -279,7 +275,6 @@ set_cpu_msg(98143);
 		return ret;
 	}
 
-set_cpu_msg(98144);
 	resp->seq_from = xli->out->h.cr_cnt;
 	if ( xli->flags & XIF_CODE_SYNC ) {
 		cm1 = s_get_cm(xli->out);
@@ -287,7 +282,6 @@ set_cpu_msg(98144);
 			s_printf(xli->out,"&cx%x;",cm1->lccode);
 		xli->flags &= ~XIF_CODE_SYNC;
 	}
-set_cpu_msg(98145);
 	if ( cmd_str )
 		s_printf(xli->out,"%ls\n",cmd_str);
 	else {
@@ -301,7 +295,6 @@ ss_printf("\n");
 		s_printf(xli->out,"\n");
 	}
 
-set_cpu_msg(98146);
 	resp->seq_to = xli->out->h.cr_cnt;
 	s_flush(xli->out);
 	_unlock_xli_out(xli);
@@ -309,7 +302,6 @@ set_cpu_msg(98146);
 	xli->result_head = resp;
 
 
-set_cpu_msg(98147);
 
 	resp->my_xli_result_next = my_xli->result_list;
 	my_xli->result_list = resp;
