@@ -74,6 +74,11 @@ typedef struct con_server {
 	int			direct_session_id;
 } CON_SERVER;
 
+
+#define LOCK_SET(s)	\
+	{(s)->lock_file = __FILE__;(s)->lock_line=__LINE__; \
+	(s)->lock_task = _get_tid();}
+
 typedef struct con_session {
 	struct con_session *	next;
 	int			type;
@@ -83,13 +88,16 @@ typedef struct con_session {
 	CON_SERVER *		server;
 	struct con_session *	sl_next;
 	int			lock;
+	char * 			lock_file;
+	int			lock_line;
+	int			lock_task;
 
 	int			flags;
 #define CSF_CLOSE		0x00000001
 #define CSF_PROXY_INIT		0x00000002
 	int			access_cnt;
 
-	int			(*pre_func)();
+	XL_SEXP *		(*pre_func)();
 	XL_SEXP *		(*pre_proxy)();
 	void			(*pre_gc)();
 	void			(*pre_free)();
@@ -642,7 +650,7 @@ CON_AGENT * a;
 int
 _set_pre_func(
 	CON_SESSION * ses,
-	int (*pre_func)(),
+	XL_SEXP * (*pre_func)(),
 	XL_SEXP * (*pre_proxy)(),
 	void (*pre_gc)(),
 	void (*pre_free)(),
@@ -692,6 +700,7 @@ CON_SESSION ** sp, * s;
 			lock_task(session_lock);
 		}
 		s->lock = 1;
+		LOCK_SET(s);
 
 		if ( s->pre_free )
 			(*s->pre_free)(s->pre_work);
@@ -718,7 +727,7 @@ typedef struct init_sexp_work {
 	XLISP_ENV *		env;
 } INIT_SEXP_WORK;
 
-int
+XL_SEXP *
 init_sexp_func(
 	CON_SESSION * ses,
 	CON_AGENT * a,
@@ -727,12 +736,7 @@ init_sexp_func(
 XL_SEXP * q;
 INIT_SEXP_WORK * w;
 	w = (INIT_SEXP_WORK*)ses->pre_work;
-	for (q = w->query  ;
-			get_type(q) != XLT_PAIR ;
-			q = cdr(q) ) {
-		remote_query(a->iid,w->env,0,car(q));
-	}
-	return 0;
+	return w->query;
 }
 
 XL_SEXP *
@@ -936,6 +940,8 @@ printf("ret C %i\n",get_tid());
 			l_string(std_cm,"client"),
 			l_string(std_cm,setagent_msg));
 
+
+
 	lock_xli_out_iid(a->iid);
 
 	ret = remote_query(a->iid,env,0,
@@ -944,8 +950,6 @@ printf("ret C %i\n",get_tid());
 			get_string(login_mode),
 			-1));
 
-
-
 	if ( get_type(ret) == XLT_ERROR ) {
 		unlock_xli_out_iid(a->iid);
 
@@ -959,6 +963,8 @@ printf("ret C %i\n",get_tid());
 				goto end;
 			goto retry;
 		}
+		close_interpreter(a->iid);
+		a->iid = 0;
 		goto end;
 	}
 	unlock_xli_out_iid(a->iid);
@@ -1172,6 +1178,7 @@ int retry_cnt;
 L_CHAR * path;
 XL_SEXP * _cmd;
 L_CHAR * agent,* login_mode, * center_cmd;
+XL_SEXP * pre_data;
 
 	if ( a_agent )
 		agent = ll_copy_str(a_agent,123);
@@ -1214,6 +1221,7 @@ L_CHAR * agent,* login_mode, * center_cmd;
 		lock_task(session_lock);
 	}
 	ses->lock = 1;
+	LOCK_SET(ses);
 	if ( login_mode ) {
 		if ( ses->login_mode )
 			d_f_ree(ses->login_mode);
@@ -1331,12 +1339,15 @@ retry:
 
 	read_lock_agent(a);
 
+	pre_data = 0;
 	lock_task(session_lock);
 	if ( _search_con_agent_initial(a,ses->id) == 0 ) {
 		unlock_task(session_lock,"_remote_session");
 
 		if ( ses->pre_func )
-			(*ses->pre_func)(ses,a,proxy_ses);
+			pre_data = (*ses->pre_func)(ses,a,proxy_ses);
+		if ( get_type(pre_data) == XLT_ERROR )
+			pre_data = 0;
 
 		lock_task(session_lock);
 		_insert_con_agent_initial(a,ses->id);
@@ -1345,7 +1356,7 @@ retry:
 
 	read_unlock_agent(a);
 
-	_cmd = cmd;
+	_cmd = append(pre_data,cmd);
 
 	for ( ; get_type(_cmd) == XLT_PAIR ; _cmd = cdr(_cmd) ) {
 
@@ -1497,20 +1508,7 @@ XL_SEXP * open_session;
 		sq,
 		-1);
 
-	_remote_session(
-			gblisp_top_env0,
-			native_proxy_session,
-			0,
-			l_string(std_cm,"xlproxy"),
-			l_string(std_cm,"user"),
-			0,
-			send_query,
-			f,
-			ln,
-			0,
-			0,
-			ses->id);
-	return 0;
+	return send_query;
 end:
 	unlock_task(session_lock,"setup_session_proxy");
 	return ret;
@@ -1586,7 +1584,22 @@ char buf[20];
 
 
 	ret = setup_session_proxy(id,f,ln);
-	if ( ret )
+	if ( get_type(ret) == XLT_ERROR )
+		return ret;
+	ret = _remote_session(
+			gblisp_top_env0,
+			native_proxy_session,
+			&native_proxy_url,
+			l_string(std_cm,"xlproxy"),
+			l_string(std_cm,"user"),
+			0,
+			ret,
+			f,
+			ln,
+			0,
+			0,
+			id);
+	if ( ret && ret->h.type == XLT_ERROR )
 		return ret;
 
 	send_query = cmd;
@@ -1758,14 +1771,18 @@ int
 set_native_proxy_direct_access(L_CHAR * server)
 {
 int ret;
+
+
 	lock_task(session_lock);
 	ret = _set_native_proxy_direct_access(server);
 	unlock_task(session_lock,"set_native_proxy");
+
+
 	return ret;
 }
 
 
-int
+XL_SEXP *
 pre_proxy_func(
 	CON_SESSION * ses,
 	CON_AGENT * a,
@@ -1777,19 +1794,22 @@ CON_SESSION * s;
 		s->flags &= ~CSF_PROXY_INIT;
 	unlock_task(session_lock,"pre_proxy_func");
 
-	setup_session_proxy(proxy_ses,0,0);
-
-	return 0;
+	return setup_session_proxy(proxy_ses,0,0);
 }
 
 int
 set_native_proxy(URL * u)
 {
 CON_SESSION * ses;
-	if ( u->server == 0 ) {
+
+
+
+	if ( u == 0 || u->server == 0 ) {
 		close_session(native_proxy_session);
 		native_proxy_session = 0;
 		free_url(&native_proxy_url);
+		zero_url(&native_proxy_url);
+
 		return 0;
 	}
 	if ( native_proxy_session == 0 )
@@ -1797,6 +1817,7 @@ CON_SESSION * ses;
 	free_url(&native_proxy_url);
 	copy_url(&native_proxy_url,u);
 
+
 	lock_task(session_lock);
 	ses = _search_con_session(native_proxy_session);
 	_set_pre_func(ses,
@@ -1806,6 +1827,7 @@ CON_SESSION * ses;
 		0,
 		0);
 	unlock_task(session_lock,"set_native_proxy");
+
 	return 0;
 }
 
@@ -1815,7 +1837,7 @@ _get_native_proxy_server_list()
 NPROXY_DIRECT_SERVER * s;
 XL_SEXP * ret;
 	ret = 0;
-	for ( s = server_list ; s ; s = s->next ) {
+	for ( s = np_server_list ; s ; s = s->next ) {
 		ret = cons(
 			List(n_get_symbol("server"),
 				n_get_string("NoProxy"),
@@ -1830,7 +1852,7 @@ XL_SEXP * ret;
 XL_SEXP *
 _get_native_proxyinfo()
 {
-	if ( native_proxy_session ) {
+	if ( native_proxy_session == 0 ) {
 		return List(n_get_symbol("proxyinfo"),
 			n_get_string("NoProxy"),
 			-1);
